summaryrefslogtreecommitdiff
path: root/libavutil/tx.h
diff options
context:
space:
mode:
authorLynne <dev@lynne.ee>2021-02-10 17:58:22 +0100
committerLynne <dev@lynne.ee>2021-02-21 17:05:16 +0100
commit5ca40d6d941bd802a7b953b3a21cd075725d5c98 (patch)
tree9403dfc23e7f2ec88baa4866f9f5d438d28600d7 /libavutil/tx.h
parentaa34e99f889af37e5c719737de1347e4985e0196 (diff)
lavu/tx: support in-place FFT transforms
This commit adds support for in-place FFT transforms. Since our internal transforms were all in-place anyway, this only changes the permutation on the input. Unfortunately, research papers were of no help here. All focused on dry hardware implementations, where permutes are free, or on software implementations where binary bloat is of no concern so storing dozen times the transforms for each permutation and version is not considered bad practice. Still, for a pure C implementation, it's only around 28% slower than the multi-megabyte FFTW3 in unaligned mode. Unlike a closed permutation like with PFA, split-radix FFT bit-reversals contain multiple NOPs, multiple simple swaps, and a few chained swaps, so regular single-loop single-state permute loops were not possible. Instead, we filter out parts of the input indices which are redundant. This allows for a single branch, and with some clever AVX512 asm, could possibly be SIMD'd without refactoring. The inplace_idx array is guaranteed to never be larger than the revtab array, and in practice only requires around log2(len) entries. The power-of-two MDCTs can be done in-place as well. And it's possible to eliminate a copy in the compound MDCTs too, however it'll be slower than doing them out of place, and we'd need to dirty the input array.
Diffstat (limited to 'libavutil/tx.h')
-rw-r--r--libavutil/tx.h14
1 files changed, 13 insertions, 1 deletions
diff --git a/libavutil/tx.h b/libavutil/tx.h
index 14d6311e02..983b5e9307 100644
--- a/libavutil/tx.h
+++ b/libavutil/tx.h
@@ -99,6 +99,18 @@ enum AVTXType {
typedef void (*av_tx_fn)(AVTXContext *s, void *out, void *in, ptrdiff_t stride);
/**
+ * Flags for av_tx_init()
+ */
+enum AVTXFlags {
+ /**
+ * Performs an in-place transformation on the input. The output parameter
+ * to av_tn_fn() will be ignored. May be unsupported or slower for some
+ * transform types.
+ */
+ AV_TX_INPLACE = 1ULL << 0,
+};
+
+/**
* Initialize a transform context with the given configuration
* (i)MDCTs with an odd length are currently not supported.
*
@@ -108,7 +120,7 @@ typedef void (*av_tx_fn)(AVTXContext *s, void *out, void *in, ptrdiff_t stride);
* @param inv whether to do an inverse or a forward transform
* @param len the size of the transform in samples
* @param scale pointer to the value to scale the output if supported by type
* @param flags currently unused
*
* @return 0 on success, negative error code on failure
*/
@@ -108,7 +120,7 @@ typedef void (*av_tx_fn)(AVTXContext *s, void *out, void *in, ptrdiff_t stride);
* @param len the size of the transform in samples
* @param scale pointer to the value to scale the output if supported by type
- * @param flags currently unused
+ * @param flags a bitmask of AVTXFlags or 0
*
* @return 0 on success, negative error code on failure